@@ -32,7 +32,7 @@ use Image::ExifTool::XMP;
 use Image::ExifTool::Canon;
 use Image::ExifTool::Nikon;
 
-$VERSION = '2.34';
+$VERSION = '2.39';
 @ISA = qw(Exporter);
 
 sub NumbersFirst;
@@ -75,6 +75,7 @@ my %formatOK = (
     int16uRev => 1,
     rational32u => 1,
     rational32s => 1,
+    pstring     => 1,
     var_string  => 1,
     var_int16u  => 1,
     var_pstr32  => 1,
@@ -108,11 +109,12 @@ meta information extracted from or written to a file.
 The tables listed below give the names of all tags recognized by ExifTool.
 },
     ExifTool2 => q{
-B<Tag ID>, B<Index> or B<Sequence> is given in the first column of each
+B<Tag ID>, B<Index#> or B<Sequence> is given in the first column of each
 table.  A B<Tag ID> is the computer-readable equivalent of a tag name, and
-is the identifier that is actually stored in the file.  An B<Index> refers
-to the location of a value when found at a fixed position within a data
-block, and B<Sequence> gives the order of values for a serial data stream.
+is the identifier that is actually stored in the file.  B<Index#> refers to
+the location of a value when found at a fixed position within a data block
+(B<#> is the multiplier for calculating a byte offset: B<1>, B<2> or B<4>).
+B<Sequence> gives the order of values for a serial data stream.
 
 A B<Tag Name> is the handle by which the information is accessed in
 ExifTool.  In some instances, more than one name may correspond to a single
@@ -213,9 +215,10 @@ GPSVersionID tag automatically if new a GPS IFD is added to an image.
 XMP stands for "Extensible Metadata Platform", an XML/RDF-based metadata
 format which is being pushed by Adobe.  Information in this format can be
 embedded in many different image file types including JPG, JP2, TIFF, GIF,
-EPS, PDF, PSD, IND, PNG, DJVU, SVG, PGF, MIFF, XCF, CRW, DNG and a variety
-of proprietary TIFF-based RAW images, as well as MOV, AVI, ASF, WMV, FLV,
-SWF and MP4 videos, and WMA and audio formats supporting ID3v2 information.
+EPS, PDF, PSD, IND, INX, PNG, DJVU, SVG, PGF, MIFF, XCF, CRW, DNG and a
+variety of proprietary TIFF-based RAW images, as well as MOV, AVI, ASF, WMV,
+FLV, SWF and MP4 videos, and WMA and audio formats supporting ID3v2
+information.
 
 The XMP B<Tag ID>'s aren't listed because in most cases they are identical
 to the B<Tag Name> (aside from differences in case).  Tags with different
@@ -269,7 +272,9 @@ tables.  For example, the C<pdfx> namespace doesn't have a predefined set of
 tag names because it is used to store application-defined PDF information,
 but this information is extracted by ExifTool.
 
-See L<http://www.adobe.com/devnet/xmp/> for the official XMP specification.
+The tables below list tags from the official XMP specification as well as
+extensions from various other sources.  See
+L<http://www.adobe.com/devnet/xmp/> for the official XMP specification.
 },
     IPTC => q{
 The tags listed below are part of the International Press Telecommunications
@@ -349,8 +354,9 @@ located in 5 different places.)
 },
     Olympus => q{
 Tags 0x0000 through 0x0103 are used by some older Olympus cameras, and are
-the same as Konica/Minolta tags.  The Olympus tags are also used for Epson
-and Agfa cameras.
+the same as Konica/Minolta tags.  These tags are also used for some models
+from other brands such as Acer, BenQ, Epson, Hitachi, HP, Maginon, Minolta,
+Pentax, Ricoh, Samsung, Sanyo, SeaLife, Sony, Supra and Vivitar.
 },
     Panasonic => q{
 These tags are used in Panasonic/Leica cameras.
@@ -426,11 +432,15 @@ and RPM).
 },
     Extra => q{
 The extra tags represent extra information extracted or generated by
-ExifTool that is not directly associated with another tag group.  The three
-writable "pseudo" tags (FileName, Directory and FileModifyDate) may be
-written without the need to rewrite the file since their values are not
-contained within the file data.  These "pseudo" tags belong to the family 1
-"System" group.
+ExifTool that is not directly associated with another tag group.  The
+B<Group> column lists the family 1 group name when reading.  Tags with a "-"
+in this column are write-only.
+
+Tags in the family 1 "System" group are referred to as "pseudo" tags because
+they don't represent real metadata in the file.  Instead, this information
+is stored in the directory structure of the filesystem.  The three writable
+"pseudo" tags (FileName, Directory and FileModifyDate) may be written
+without modifying the file itself.
 },
     Composite => q{
 The values of the composite tags are B<Derived From> the values of other
@@ -455,7 +465,7 @@ L<Image::ExifTool::BuildTagLookup|Image::ExifTool::BuildTagLookup>.
 
 ~head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -638,8 +648,9 @@ sub new
             $$table{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData or
             $$table{PROCESS_PROC} eq \&Image::ExifTool::Nikon::ProcessNikonEncrypted));
         if ($$vars{ID_LABEL} or $processBinaryData) {
+            my $s = $$table{FORMAT} ? Image::ExifTool::FormatSize($$table{FORMAT}) || 1 : 1;
             $binaryTable = 1;
-            $id{$tableName} = $$vars{ID_LABEL} || 'Index';
+            $id{$tableName} = $$vars{ID_LABEL} || "Index$s";
         } elsif ($isIPTC and $$table{PROCESS_PROC}) { #only the main IPTC table has a PROCESS_PROC
             $id{$tableName} = 'Record';
         } elsif (not $noID) {
@@ -767,7 +778,7 @@ TagID:  foreach $tagID (@keys) {
                 if ($$tagInfo{Notes}) {
                     my $note = $$tagInfo{Notes};
                     # remove leading/trailing blank lines
-                    $note =~ s/(^\s+|\s+$)//g;
+                    $note =~ s/^\s+//; $note =~ s/\s+$//;
                     # remove leading/trailing spaces on each line
                     $note =~ s/(^[ \t]+|[ \t]+$)//mg;
                     push @values, "($note)";
@@ -780,7 +791,12 @@ TagID:  foreach $tagID (@keys) {
                     }
                 }
                 my $writeGroup;
-                $writeGroup = $$tagInfo{WriteGroup};
+                if ($short eq 'Extra') {
+                    my @g = $exifTool->GetGroup($tagInfo);
+                    $writeGroup = $$tagInfo{WriteOnly} ? '-' : $g[1];
+                } else {
+                    $writeGroup = $$tagInfo{WriteGroup};
+                }
                 unless ($writeGroup) {
                     $writeGroup = $$table{WRITE_GROUP} if $writable;
                     $writeGroup = '-' unless $writeGroup;
@@ -1470,6 +1486,8 @@ sub GetTableOrder()
     }
     # tweak the table order
     my %tweakOrder = (
+      # this    => comes after this
+      # -------    -----------------
         JPEG    => '-',     # JPEG comes first
         IPTC    => 'Exif',  # put IPTC after EXIF,
         GPS     => 'XMP',   # etc...
@@ -1498,8 +1516,10 @@ sub GetTableOrder()
        'Nikon::CameraSettingsD300' => 'Nikon::ShotInfoD300b',
        'Pentax::LensData' => 'Pentax::LensInfo2',
        'Sony::SRF2' => 'Sony::SRF',
-       'Samsung::INFO' => 'Samsung::Type2', # (necessary because Samsung doesn't have a main table)
-       'Samsung::MP4' => 'Samsung::INFO', # (necessary because Samsung doesn't have a main table)
+       'Samsung::PictureWizard' => 'Samsung::Type2', # (necessary because Samsung doesn't have a main table)
+       'Samsung::INFO' => 'Samsung::PictureWizard', # (ditto)
+       'Samsung::MP4' => 'Samsung::INFO', # (ditto)
+       'Samsung::Thumbnail' => 'Samsung::MP4', # (ditto)
     );
     my @tweak = sort keys %tweakOrder;
     while (@tweak) {
@@ -1732,7 +1752,7 @@ sub WriteTagNames($$)
             my $notes = $$printConv{Notes};
             if ($notes) {
                 # remove unnecessary whitespace
-                $notes =~ s/(^\s+|\s+$)//g;
+                $notes =~ s/^\s+//; $notes =~ s/\s+$//;
                 $notes =~ s/(^[ \t]+|[ \t]+$)//mg;
             }
             my $head = $tableName;
@@ -1755,7 +1775,7 @@ sub WriteTagNames($$)
                 }
                 $wid = length($tableName)+7 if $wid < length($tableName)+7;
                 # print in multiple columns if there is room
-                my $cols = int(80 / ($wid + 4));
+                my $cols = int(110 / ($wid + 4));
                 $cols = 1 if $cols < 1 or $cols > @keys or @keys < 4;
                 my $rows = int((scalar(@keys) + $cols - 1) / $cols);
                 my ($r, $c);
@@ -1847,7 +1867,7 @@ sub WriteTagNames($$)
         my $podIdLen = $self->{LONG_ID}->{$tableName};
         if ($notes) {
             # remove unnecessary whitespace
-            $notes =~ s/(^\s+|\s+$)//g;
+            $notes =~ s/^\s+//; $notes =~ s/\s+$//;
             $notes =~ s/(^[ \t]+|[ \t]+$)//mg;
             if ($notes =~ /leading '(.*?_)' which/) {
                 $prefix = $1;
@@ -1882,9 +1902,10 @@ sub WriteTagNames($$)
             $wTag += 9;
             $hid = '';
         }
-        if ($short eq 'EXIF') {
+        if ($short eq 'EXIF' or $short eq 'Extra') {
             $derived = '<th>Group</th>';
             $showGrp = 1;
+            $wGrp += 8 if $short eq 'Extra';    # tweak Group column width for "Extra" tags
             $wTag -= $wGrp + 1;
         }
         my $head = ($short =~ / /) ? 'head3' : 'head2';
@@ -2233,7 +2254,7 @@ WriteTagNames().
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
