@@ -9,7 +9,6 @@
 #               2) Christian Koller private communication (20D)
 #               3) Rainer Honle private communication (5D)
 #               4) David Pitcher private communication (1DmkIII firmware upgrade)
-#               5) Laurent Clevy private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonCustom;
@@ -20,7 +19,7 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Canon;
 use Image::ExifTool::Exif;
 
-$VERSION = '1.39';
+$VERSION = '1.42';
 
 sub ProcessCanonCustom($$$);
 sub ProcessCanonCustom2($$$);
@@ -1217,8 +1216,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0101 => [
         {
             Name => 'ExposureLevelIncrements',
-            Notes => '1DmkIII and 1DmkIV',
             Condition => '$$self{Model} =~ /\b1Ds?\b/',
+            Notes => '1DmkIII and 1DmkIV',
             PrintConv => {
                 0 => '1/3-stop set, 1/3-stop comp.',
                 1 => '1-stop set, 1/3-stop comp.',
@@ -1286,7 +1285,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     },
     0x0105 => {
         Name => 'AEBSequence',
-        Notes => 'value of 2 not used by 40D, 50D, 5DmkII and 7D',
+        Notes => 'value of 2 not used by 40D, 50D, 60D, 5DmkII and 7D',
         PrintConv => {
             0 => '0,-,+',
             1 => '-,0,+',
@@ -1313,7 +1312,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     },
     0x0108 => {
         Name => 'SafetyShift',
-        Notes => 'value of 2 not used by 40D, 50D, 5DmkII and 7D',
+        Notes => 'value of 2 not used by 40D, 50D, 60D, 5DmkII and 7D',
         PrintConv => {
             0 => 'Disable',
             1 => 'Enable (Tv/Av)',
@@ -1419,8 +1418,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{Model} =~ /\b(50D|7D)\b/',
-            Notes => '50D and 7D',
+            Condition => '$$self{Model} =~ /\b(50D|60D|7D)\b/',
+            Notes => '50D, 60D and 7D',
             PrintConv => {
                 0 => 'Auto',
                 1 => '1/250-1/60 Auto',
@@ -1438,17 +1437,16 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{Model} =~ /\b(5D Mark II|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '5D Mark II, 500D and 550D',
+            Condition => '$$self{Model} =~ /\bEOS-1Ds? Mark III\b/',
+            Notes => '1D Mark III and 1Ds Mark III',
             PrintConv => {
                 0 => 'Auto',
-                1 => '1/200-1/60 Auto',
-                2 => '1/200 Fixed',
+                1 => '1/300 Fixed',
             },
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{Model} =~ /\b(EOS-1D Mark IV)\b/',
+            Condition => '$$self{Model} =~ /\bEOS-1D Mark IV\b/',
             Notes => '1D Mark IV',
             PrintConv => {
                 0 => 'Auto',
@@ -1458,10 +1456,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'FlashSyncSpeedAv',
-            Notes => '1D Mark III',
+            Notes => '5D Mark II, 500D, 550D, 600D and 1100D',
             PrintConv => {
                 0 => 'Auto',
-                1 => '1/300 Fixed',
+                1 => '1/200-1/60 Auto',
+                2 => '1/200 Fixed',
             },
         },
     ],
@@ -1487,8 +1486,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0202 => [
         {
             Name => 'HighISONoiseReduction',
-            Condition => '$$self{Model} =~ /\b(50D|5D Mark II|7D|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '50D, 500D, 550D, 5DmkII and 7D',
+            Condition => q{
+                $$self{Model} =~ /\b(50D|60D|5D Mark II|7D|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/ or
+                $$self{Model} =~ /\b(600D|T3i|Kiss X5|1100D|T3|Kiss X50)\b/
+            },
+            Notes => '50D, 60D, 500D, 550D, 600D, 1100D, 5DmkII and 7D',
             PrintConv => {
                 0 => 'Standard',
                 1 => 'Low',
@@ -1511,7 +1513,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             Name => 'AutoLightingOptimizer',
             Condition => '$$self{Model} =~ /\b(50D|5D Mark II|500D|T1i|Kiss X3|1D Mark IV)\b/',
             Notes => '50D, 500D, 5DmkII and 1DmkIV',
-            PrintConv => { #5
+            PrintConv => {
                 0 => 'Standard',
                 1 => 'Low',
                 2 => 'Strong',
@@ -1733,32 +1735,45 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x050e => [
         {
             Name => 'AFAssistBeam',
+            Condition => '$$self{Model} =~ /\b1D Mark IV\b/',
             Notes => '1D Mark IV',
             PrintConv => {
                 0 => 'Emits',
                 1 => 'Does not emit',
-                2 => 'IR AF assist bean only',
+                2 => 'IR AF assist beam only',
             },
         },
         {
             Name => 'AFAssistBeam',
-            Notes => 'values 2-3 not used by 1DmkIII or 5DmkII, value 3 new for 7D',
+            Notes => 'other models; values 2-3 not used by 1DmkIII or 5DmkII, value 3 new for 7D',
             PrintConv => {
                 0 => 'Emits',
                 1 => 'Does not emit',
                 2 => 'Only ext. flash emits',
-                3 => 'IR AF assist bean only', # new for 7D
+                3 => 'IR AF assist beam only', # new for 7D
             },
         },
     ],
-    0x050f => { # new for 40D
-        Name => 'AFPointSelectionMethod',
-        PrintConv => {
-            0 => 'Normal',
-            1 => 'Multi-controller direct',
-            2 => 'Quick Control Dial direct',
+    0x050f => [ # new for 40D
+        {
+            Name => 'AFPointSelectionMethod',
+            Condition => '$$self{Model} !~ /\b60D\b/',
+            Notes => '40D, 50D and 5DmkII',
+            PrintConv => {
+                0 => 'Normal',
+                1 => 'Multi-controller direct',
+                2 => 'Quick Control Dial direct',
+            },
         },
-    },
+        {
+            Name => 'AFPointSelectionMethod',
+            Notes => '60D',
+            PrintConv => {
+                0 => 'AF point button: Activate AF Sel; Rear dial: Select AF points',
+                1 => 'AF point button: Auto selection; Rear dial: Manual selection',
+            },
+        },
+    ],
     0x0510 => [ # new for 40D
         {
             Name => 'VFDisplayIllumination',
@@ -1874,8 +1889,11 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0701 => [
         {
             Name => 'Shutter-AELock',
-            Condition => '$$self{Model} =~ /\b(1000D|XS|Kiss F|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/',
-            Notes => '500D, 550D and 1000D',
+            Condition => q{
+                $$self{Model} =~ /\b(1000D|XS|Kiss F|500D|T1i|Kiss X3|550D|T2i|Kiss X4)\b/ or
+                $$self{Model} =~ /\b(600D|T3i|Kiss X5|1100D|T3|Kiss X50)\b/
+            },
+            Notes => '500D, 550D, 600D, 1000D and 1100D',
             PrintConv => {
                 0 => 'AF/AE lock',
                 1 => 'AE lock/AF',
@@ -1883,6 +1901,18 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 3 => 'AE/AF, No AE lock',
             },
         },
+        {
+            Name => 'AFAndMeteringButtons',
+            Condition => '$$self{Model} =~ /\b60D\b/',
+            Notes => '60D',
+            PrintConv => {
+                0 => 'Metering start',
+                1 => 'Metering + AF start',
+                2 => 'AE lock',
+                3 => 'AF stop',
+                4 => 'No function',
+            },
+        },
         {
             Name => 'ShutterButtonAFOnButton',
             Notes => 'other models',
@@ -1926,8 +1956,21 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
         {
             Name => 'SetButtonWhenShooting',
-            Condition => '$$self{Model} =~ /\b(450D|XSi|Kiss X2|550D|T2i|Kiss X4)\b/',
-            Notes => '450D and 550D; value of 5 is new for 550D',
+            Condition => '$$self{Model} =~ /\b60D\b/',
+            Notes => '60D',
+            PrintConv => {
+                0 => 'Normal (disabled)',
+                1 => 'Image quality',
+                2 => 'Picture style',
+                3 => 'White balance',
+                4 => 'Flash exposure compensation',
+                5 => 'Viewfinder leveling gauge',
+            },
+        },
+        {
+            Name => 'SetButtonWhenShooting',
+            Condition => '$$self{Model} =~ /\b(450D|XSi|Kiss X2|550D|T2i|Kiss X4|600D|T3i|Kiss X5)\b/',
+            Notes => '450D, 550D and 600D; value of 5 is new for 550D',
             PrintConv => {
                 0 => 'Normal (disabled)',
                 1 => 'Image quality',
@@ -1937,6 +1980,19 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 5 => 'ISO speed',
             },
         },
+        {
+            Name => 'SetButtonWhenShooting',
+            Condition => '$$self{Model} =~ /\b(1100D|T3|Kiss X50)\b/',
+            Notes => '1100D',
+            PrintConv => {
+                0 => 'Normal (disabled)',
+                1 => 'Image quality',
+                2 => 'Flash exposure compensation',
+                3 => 'LCD monitor On/Off',
+                4 => 'Menu display',
+                5 => 'Depth-of-field preview',
+            },
+        },
         {
             Name => 'SetButtonWhenShooting',
             Condition => '$$self{Model} =~ /\b(1000D|XS|Kiss F)\b/',
@@ -2040,12 +2096,19 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             1 => 'Quick start (FEL button)',
         },
     },
+    0x070e => { # new for 1100D
+        Name => 'FlashButtonFunction',
+        PrintConv => {
+            0 => 'Raise built-in flash',
+            1 => 'ISO speed',
+        },
+    },
     #### 4b) Others
     0x080b => [
         {
             Name => 'FocusingScreen',
-            Condition => '$$self{Model} =~ /\b(40D|50D)\b/',
-            Notes => '40D and 50D',
+            Condition => '$$self{Model} =~ /\b(40D|50D|60D)\b/',
+            Notes => '40D, 50D and 60D',
             PrintConv => {
                 0 => 'Ef-A',
                 1 => 'Ef-D',
@@ -2197,15 +2260,15 @@ sub ProcessCanonCustom2($$$)
                 my $tagInfo = $$newTags{$tag};
                 next unless $tagInfo;
                 my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                next unless Image::ExifTool::IsOverwriting($nvHash, $val);
-                my $newVal = Image::ExifTool::GetNewValues($nvHash);
+                next unless $exifTool->IsOverwriting($nvHash, $val);
+                my $newVal = $exifTool->GetNewValues($nvHash);
                 next unless defined $newVal;    # can't delete from a custom table
                 WriteValue($newVal, 'int32s', $num, $dataPt, $recPos);
                 $exifTool->VerboseValue("- CanonCustom:$$tagInfo{Name}", $val);
                 $exifTool->VerboseValue("+ CanonCustom:$$tagInfo{Name}", $newVal);
                 ++$exifTool->{CHANGED};
             } else {
-                # extract the value
+                # save extracted tag
                 my $oldInfo = $$tagTablePtr{$tag};
                 $exifTool->HandleTag($tagTablePtr, $tag, $val,
                     Index  => $i,
@@ -2308,7 +2371,6 @@ sub WriteCanonCustom($$$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || length($$dataPt) - $dirStart;
     my $dirName = $$dirInfo{DirName};
-    my $verbose = $exifTool->Options('Verbose');
     my $newData = substr($$dataPt, $dirStart, $dirLen) or return undef;
     $dataPt = \$newData;
 
@@ -2327,8 +2389,8 @@ sub WriteCanonCustom($$$)
         next unless $tagInfo;
         my $nvHash = $exifTool->GetNewValueHash($tagInfo);
         $val = ($val & 0xff);
-        next unless Image::ExifTool::IsOverwriting($nvHash, $val);
-        my $newVal = Image::ExifTool::GetNewValues($nvHash);
+        next unless $exifTool->IsOverwriting($nvHash, $val);
+        my $newVal = $exifTool->GetNewValues($nvHash);
         next unless defined $newVal;    # can't delete from a custom table
         Set16u(($newVal & 0xff) + ($tag << 8), $dataPt, $pos);
         $exifTool->VerboseValue("- $dirName:$$tagInfo{Name}", $val);
@@ -2360,7 +2422,7 @@ Image::ExifTool to read this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
