@@ -17,7 +17,7 @@ use Image::ExifTool::Exif;
 use Image::ExifTool::MakerNotes;
 use Image::ExifTool::CanonRaw;
 
-$VERSION = '1.14';
+$VERSION = '1.18';
 
 sub ProcessOriginalRaw($$$);
 sub ProcessAdobeData($$$);
@@ -56,7 +56,7 @@ sub WriteAdobeStuff($$$);
 
         The maker notes ('MakN') are processed by ExifTool, but some information may
         have been lost by the Adobe DNG Converter.  This is because the Adobe DNG
-        Converter (as of version 4.3) doesn't properly handle information referenced
+        Converter (as of version 6.3) doesn't properly handle information referenced
         from inside the maker notes that lies outside the original maker notes
         block.  This information is lost when only the maker note block is copied to
         the DNG image.   While this doesn't effect all makes of cameras, it is a
@@ -168,8 +168,9 @@ sub ProcessOriginalRaw($$$)
             $pos + $hdrLen > $end and $err = '', last;
             my $tag = $$tagInfo{Name};
             # only extract this information if requested (because it takes time)
-            if ($exifTool->{OPTIONS}->{Binary} or
-                $exifTool->{REQ_TAG_LOOKUP}->{lc($tag)})
+            my $lcTag = lc $tag;
+            if (($exifTool->{OPTIONS}{Binary} and not $exifTool->{EXCL_TAG_LOOKUP}{$lcTag}) or
+                $exifTool->{REQ_TAG_LOOKUP}{$lcTag})
             {
                 unless (eval 'require Compress::Zlib') {
                     $err = 'Install Compress::Zlib to extract compressed images';
@@ -418,8 +419,8 @@ sub ProcessAdobeCRW($$$)
                     }
                 } elsif ($$newTags{$tagID}) {
                     my $nvHash = $exifTool->GetNewValueHash($tagInfo);
-                    if (Image::ExifTool::IsOverwriting($nvHash, $val)) {
-                        my $newVal = Image::ExifTool::GetNewValues($nvHash);
+                    if ($exifTool->IsOverwriting($nvHash, $val)) {
+                        my $newVal = $exifTool->GetNewValues($nvHash);
                         my $verboseVal;
                         $verboseVal = $newVal if $verbose > 1;
                         # convert to specified format if necessary
@@ -608,7 +609,6 @@ sub ProcessAdobeIFD($$$)
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart};
     my $dataPos = $$dirInfo{DataPos};
-    my $base = $$dirInfo{Base};
 
     return 0 if $$dirInfo{DirLen} < 4;
     my $dataOrder = substr($$dataPt, $pos, 2);
@@ -755,7 +755,7 @@ sub ProcessAdobeMakN($$$)
         $exifTool->ProcessDirectory(\%subdirInfo, $subTable, $$subdir{ProcessProc});
         # extract maker notes as a block if specified
         if ($exifTool->Options('MakerNotes') or
-            $exifTool->{REQ_TAG_LOOKUP}->{lc($$tagInfo{Name})})
+            $exifTool->{REQ_TAG_LOOKUP}{lc($$tagInfo{Name})})
         {
             my $val;
             if ($$tagInfo{MakerNotes}) {
@@ -765,7 +765,9 @@ sub ProcessAdobeMakN($$$)
                 $subdirInfo{DirLen}   = $dirLen;
                 # rebuild the maker notes to identify all offsets that require fixing up
                 $val = Image::ExifTool::Exif::RebuildMakerNotes($exifTool, $subTable, \%subdirInfo);
-                defined $val or $exifTool->Warn('Error rebuilding maker notes (may be corrupt)');
+                if (not defined $val and $dirLen > 4) {
+                    $exifTool->Warn('Error rebuilding maker notes (may be corrupt)');
+                }
             } else {
                 # extract this directory as a block if specified
                 return 1 unless $$tagInfo{Writable};
@@ -811,7 +813,7 @@ information in DNG (Digital Negative) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
