@@ -135,7 +135,7 @@ sub ProcessBZZ($$$);
         any tags that exist even if they don't appear here.  The DjVu v3
         documentation endorses tags borrowed from two standards: 1) BibTeX
         bibliography system tags (all lowercase Tag ID's in the table below), and 2)
-        PDF DocInfo tags (uppercase Tag ID's).
+        PDF DocInfo tags (capitalized Tag ID's).
     },
     # BibTeX tags (ref http://en.wikipedia.org/wiki/BibTeX)
     address     => { Groups => { 2 => 'Location' } },
@@ -248,7 +248,6 @@ sub ProcessAnt($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
-    my $unknown = $exifTool->Options('Unknown');
 
     # quick pre-scan to check for metadata or XMP
     return 1 unless $$dataPt =~ /\(\s*(metadata|xmp)[\s("]/s;
@@ -349,7 +348,7 @@ Image::ExifTool::AIFF.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
