@@ -10,6 +10,8 @@
 #               3) http://ffmpeg.org/
 #               4) US Patent 2009/0052875 A1
 #               5) European Patent (EP2 051 528A1) application no. 07792522.0 filed 08.08.2007
+#               6) Dave Nicholson private communication
+#               7) http://www.freepatentsonline.com/20050076039.pdf
 #
 # Glossary:     RBSP = Raw Byte Sequence Payload
 #------------------------------------------------------------------------------
@@ -17,21 +19,31 @@
 package Image::ExifTool::H264;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION %convMake);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.00';
+$VERSION = '1.07';
 
 sub ProcessSEI($$);
 
 my $parsePictureTiming; # flag to enable parsing of picture timing information (test only)
 
+# lookup for camera manufacturer name
+%convMake = (
+    0x0103 => 'Panasonic',
+    0x0108 => 'Sony',
+    0x1011 => 'Canon',
+);
+
 # information extracted from H.264 video streams
 %Image::ExifTool::H264::Main = (
     GROUPS => { 2 => 'Video' },
     VARS => { NO_ID => 1 },
-    NOTES => 'Tags extracted from H.264 video streams.',
+    NOTES => q{
+        Tags extracted from H.264 video streams.  The metadata for AVCHD videos is
+        stored in this stream.
+    },
     ImageWidth => { },
     ImageHeight => { },
     MDPM => { SubDirectory => { TagTable => 'Image::ExifTool::H264::MDPM' } },
@@ -39,20 +51,41 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
 
 # H.264 Supplemental Enhancement Information User Data (ref PH/4)
 %Image::ExifTool::H264::MDPM = (
-    GROUPS => { 2 => 'Location' },
+    GROUPS => { 2 => 'Camera' },
     PROCESS_PROC => \&ProcessSEI,
+    TAG_PREFIX => 'MDPM',
     NOTES => q{
         The following tags are decoded from the Modified Digital Video Pack Metadata
         (MDPM) of the unregistered user data with UUID
         17ee8c60f84d11d98cd60800200c9a66 in the H.264 Supplemental Enhancement
-        Information (SEI).  [Yes, this description is confusing, but nothing
-        compared to the challenge of actually decoding the data!]  This information
+        Information (SEI).  I<[Yes, this description is confusing, but nothing
+        compared to the challenge of actually decoding the data!]>  This information
         may exist at regular intervals through the entire video, but only the first
-        occurrence is extracted unless the ExtractEmbedded option is used (in which
-        case subsequent occurrences are extracted as sub-documents).
+        occurrence is extracted unless the ExtractEmbedded (-ee) option is used (in
+        which case subsequent occurrences are extracted as sub-documents).
     },
-    # 0x13 - TimeCode
-    # 0x14 - BinaryGrup
+    # (Note: all these are explained in IEC 61834-4, but it costs money so it is useless to me)
+    # 0x00 - ControlCassetteID (ref 7)
+    # 0x01 - ControlTapeLength (ref 7)
+    # 0x02 - ControlTimerActDate (ref 7)
+    # 0x03 - ControlTimerACS_S_S (ref 7)
+    # 0x04-0x05 - ControlPR_StartPoint (ref 7)
+    # 0x06 - ControlTagIDNoGenre (ref 7)
+    # 0x07 - ControlTopicPageHeader (ref 7)
+    # 0x08 - ControlTextHeader (ref 7)
+    # 0x09 - ControlText (ref 7)
+    # 0x0a-0x0b - ControlTag (ref 7)
+    # 0x0c - ControlTeletextInfo (ref 7)
+    # 0x0d - ControlKey (ref 7)
+    # 0x0e-0x0f - ControlZoneEnd (ref 7)
+    # 0x10 - TitleTotalTime (ref 7)
+    # 0x11 - TitleRemainTime (ref 7)
+    # 0x12 - TitleChapterTotalNo (ref 7)
+    # 0x13 - TitleTimecode
+    # 0x14 - TitleBinaryGroup
+    # 0x15 - TitleCassetteNo (ref 7)
+    # 0x16-0x17 - TitleSoftID (ref 7)
+    # (0x18,0x19 listed as TitleTextHeader/TitleText by ref 7)
     0x18 => {
         Name => 'DateTimeOriginal',
         Description => 'Date/Time Original',
@@ -73,6 +106,18 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
         },
         PrintConv => '$self->ConvertDateTime($val)',
     },
+    # 0x1a-0x1b - TitleStart (ref 7)
+    # 0x1c-0x1d - TitleReelID (ref 7)
+    # 0x1e-0x1f - TitleEnd (ref 7)
+    # 0x20 - ChapterTotalTime (ref 7)
+    # 0x42 - ProgramRecDTime (ref 7)
+    # 0x50/0x60 - (AAUX/VAUX)Source (ref 7)
+    # 0x51/0x61 - (AAUX/VAUX)SourceControl (ref 7)
+    # 0x52/0x62 - (AAUX/VAUX)RecDate (ref 7)
+    # 0x53/0x63 - (AAUX/VAUX)RecTime (ref 7)
+    # 0x54/0x64 - (AAUX/VAUX)BinaryGroup (ref 7)
+    # 0x55/0x65 - (AAUX/VAUX)ClosedCaption (ref 7)
+    # 0x56/0x66 - (AAUX/VAUX)TR (ref 7)
     0x70 => { # ConsumerCamera1
         Name => 'Camera1',
         SubDirectory => { TagTable => 'Image::ExifTool::H264::Camera1' },
@@ -92,23 +137,25 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     # 0x7e Knee
     0x7f => { # Shutter
         Name => 'Shutter',
-        SubDirectory => { TagTable => 'Image::ExifTool::H264::Shutter' },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::H264::Shutter',
+            ByteOrder => 'LittleEndian', # weird
+        },
     },
     0xa0 => {
         Name => 'ExposureTime',
         Format => 'rational32u',
-        Groups => { 2 => 'Camera' },
+        Groups => { 2 => 'Image' },
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
     },
     0xa1 => {
         Name => 'FNumber',
         Format => 'rational32u',
-        Groups => { 2 => 'Camera' },
+        Groups => { 2 => 'Image' },
     },
     0xa2 => {
         Name => 'ExposureProgram',
         Format => 'int32u', # (guess)
-        Groups => { 2 => 'Camera' },
         PrintConv => {
             0 => 'Not Defined',
             1 => 'Manual',
@@ -130,19 +177,17 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
         Name => 'ExposureCompensation',
         Format => 'rational32s',
         Groups => { 2 => 'Image' },
-        PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+        PrintConv => 'Image::ExifTool::Exif::PrintFraction($val)',
     },
     0xa5 => {
         Name => 'MaxApertureValue',
         Format => 'rational32u',
-        Groups => { 2 => 'Camera' },
         ValueConv => '2 ** ($val / 2)',
         PrintConv => 'sprintf("%.1f",$val)',
     },
     0xa6 => {
         Name => 'Flash',
         Format => 'int32u', # (guess)
-        Groups => { 2 => 'Camera' },
         Flags => 'PrintHex',
         SeparateTable => 'EXIF Flash',
         PrintConv =>  \%Image::ExifTool::Exif::flash,
@@ -159,7 +204,6 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xa8 => {
         Name => 'WhiteBalance',
         Format => 'int32u', # (guess)
-        Groups => { 2 => 'Camera' },
         Priority => 0,
         PrintConv => {
             0 => 'Auto',
@@ -169,13 +213,11 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xa9 => {
         Name => 'FocalLengthIn35mmFormat',
         Format => 'rational32u',
-        Groups => { 2 => 'Camera' },
         PrintConv => '"$val mm"',
     },
     0xaa => {
         Name => 'SceneCaptureType',
         Format => 'int32u', # (guess)
-        Groups => { 2 => 'Camera' },
         PrintConv => {
             0 => 'Standard',
             1 => 'Landscape',
@@ -188,11 +230,13 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
         Name => 'GPSVersionID',
         Format => 'int8u',
         Count => 4,
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => '$val =~ tr/ /./; $val',
     },
     0xb1 => {
         Name => 'GPSLatitudeRef',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             N => 'North',
             S => 'South',
@@ -201,6 +245,7 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xb2 => {
         Name => 'GPSLatitude',
         Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         Notes => 'combined with tags 0xb3 and 0xb4',
         Combine => 2,   # combine the next 2 tags (0xb2=deg, 0xb3=min, 0xb4=sec)
         ValueConv => 'Image::ExifTool::GPS::ToDegrees($val)',
@@ -209,6 +254,7 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xb5 => {
         Name => 'GPSLongitudeRef',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             E => 'East',
             W => 'West',
@@ -217,6 +263,7 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xb6 => {
         Name => 'GPSLongitude',
         Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         Combine => 2,   # combine the next 2 tags (0xb6=deg, 0xb7=min, 0xb8=sec)
         Notes => 'combined with tags 0xb7 and 0xb8',
         ValueConv => 'Image::ExifTool::GPS::ToDegrees($val)',
@@ -225,16 +272,22 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xb9 => {
         Name => 'GPSAltitudeRef',
         Format => 'int32u', # (guess)
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         ValueConv => '$val ? 1 : 0', # because I'm not sure about the Format
         PrintConv => {
             0 => 'Above Sea Level',
             1 => 'Below Sea Level',
         },
     },
-    0xba => { Name => 'GPSAltitude', Format => 'rational32u' },
+    0xba => {
+        Name => 'GPSAltitude',
+        Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
+    },
     0xbb => {
         Name => 'GPSTimeStamp',
         Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Time' },
         Combine => 2,    # the next tags (0xbc/0xbd) contain the minutes/seconds
         Notes => 'combined with tags 0xbc and 0xbd',
         ValueConv => 'Image::ExifTool::GPS::ConvertTimeStamp($val)',
@@ -242,6 +295,7 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xbe => {
         Name => 'GPSStatus',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             A => 'Measurement Active',
             V => 'Measurement Void',
@@ -250,6 +304,7 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     0xbf => {
         Name => 'GPSMeasureMode',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             2 => '2-Dimensional Measurement',
             3 => '3-Dimensional Measurement',
@@ -259,38 +314,55 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
         Name => 'GPSDOP',
         Description => 'GPS Dilution Of Precision',
         Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
     },
     0xc1 => {
         Name => 'GPSSpeedRef',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             K => 'km/h',
             M => 'mph',
             N => 'knots',
         },
     },
-    0xc2 => { Name => 'GPSSpeed', Format => 'rational32u' },
+    0xc2 => {
+        Name => 'GPSSpeed',
+        Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
+    },
     0xc3 => {
         Name => 'GPSTrackRef',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             M => 'Magnetic North',
             T => 'True North',
         },
     },
-    0xc4 => { Name => 'GPSTrack', Format => 'rational32u' },
+    0xc4 => {
+        Name => 'GPSTrack',
+        Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
+    },
     0xc5 => {
         Name => 'GPSImgDirectionRef',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         PrintConv => {
             M => 'Magnetic North',
             T => 'True North',
         },
     },
-    0xc6 => { Name => 'GPSImgDirection', Format => 'rational32u' },
+    0xc6 => {
+        Name => 'GPSImgDirection',
+        Format => 'rational32u',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
+    },
     0xc7 => {
         Name => 'GPSMapDatum',
         Format => 'string',
+        Groups => { 1 => 'GPS', 2 => 'Location' },
         Combine => 1,    # the next tag (0xc8) contains the rest of the string
         Notes => 'combined with tag 0xc8',
     },
@@ -301,7 +373,28 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     },
     # 0xe1-0xef - MakerOption
     # 0xe1 - val: 0x01000670,0x01000678,0x06ffffff,0x01ffffff,0x01000020,0x01000400...
-    # 0xe2-0xe8 - val: 0x00000000
+    # 0xe2-0xe8 - val: 0x00000000 in many samples
+    0xe1 => { #6
+        Name => 'RecInfo',
+        Condition => '$$self{Make} eq "Canon"',
+        Notes => 'Canon only',
+        SubDirectory => { TagTable => 'Image::ExifTool::H264::RecInfo' },
+    },
+    0xe4 => { #PH
+        Name => 'Model',
+        Condition => '$$self{Make} eq "Sony"',
+        Description => 'Camera Model Name',
+        Notes => 'Sony cameras only, combined with tags 0xe5 and 0xe6',
+        Format => 'string',
+        Combine => 2, # (not sure about 0xe6, but include it just in case)
+        RawConv => '$val eq "" ? undef : $val',
+    },
+    0xee => { #6 (HFS200)
+        Name => 'FrameInfo',
+        Condition => '$$self{Make} eq "Canon"',
+        Notes => 'Canon only',
+        SubDirectory => { TagTable => 'Image::ExifTool::H264::FrameInfo' },
+    },
 );
 
 # ConsumerCamera1 information (ref PH)
@@ -313,19 +406,22 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     FIRST_ENTRY => 0,
     0 => {
         Name => 'ApertureSetting',
+        PrintHex => 1,
         PrintConv => {
             0xff => 'Auto',
             0xfe => 'Closed',
             OTHER => sub { sprintf('%.1f', 2 ** (($_[0] & 0x3f) / 8)) },
         },
     },
-    1.1 => {
+    1 => {
         Name => 'Gain',
         Mask => 0x0f,
+        # (0x0f would translate to 42 dB, but this value is used by the Sony
+        #  HXR-NX5U for any out-of-range value such as -6 dB or "hyper gain" - PH)
         ValueConv => '($val - 1) * 3',
-        PrintConv => '"$val dB"',
+        PrintConv => '$val==42 ? "Out of range" : "$val dB"',
     },
-    1.2 => {
+    1.1 => {
         Name => 'ExposureProgram',
         Mask => 0xf0,
         ValueConv => '$val == 0xf0 ? undef : $val',
@@ -367,11 +463,13 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     FIRST_ENTRY => 0,
     1 => {
         Name => 'ImageStabilization',
-        # no stabilization feature if whole byte is 0xff
-        ValueConv => '$val == 0xff ? 0 : $val & 0x10',
         PrintConv => {
-            0x00 => 'Off',
-            0x10 => 'On',
+            0 => 'Off',
+            0xff => 'n/a',
+            OTHER => sub {
+                my $val = shift;
+                sprintf("%s (0x%.2x)", $val & 0x10 ? "On" : "Off", $val);
+            },
         },
     },
 );
@@ -379,19 +477,18 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
 # camera info 0x7f (ref PH)
 %Image::ExifTool::H264::Shutter = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    GROUPS => { 2 => 'Camera' },
+    GROUPS => { 2 => 'Image' },
     TAG_PREFIX => 'Shutter',
     PRINT_CONV => 'sprintf("0x%.2x",$val)',
     FIRST_ENTRY => 0,
-  # this worked for the HDR-CX500, but not for other models
-  # --> more work is needed to decode this
-  #  2.1 => {
-  #      Name => 'ExposureTime',
-  #      Mask => 0xf0,
-  #      RawConv => '$val == 0xf0 ? undef : $val',
-  #      ValueConv => '$val / 32000',
-  #      PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
-  #  },
+    FORMAT => 'int16u',
+    1.1 => { #6
+        Name => 'ExposureTime',
+        Mask => 0x7fff, # (what is bit 0x8000 for?)
+        RawConv => '$val == 0x7fff ? undef : $val', #7
+        ValueConv => '$val / 33640', #PH (conversion factor determined empirically)
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+    },
 );
 
 # camera info 0xe0 (ref PH)
@@ -399,19 +496,50 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Camera' },
     FORMAT => 'int16u',
+    FIRST_ENTRY => 0,
     0 => {
         Name => 'Make',
         PrintHex => 1,
-        PrintConv => {
-            0x0108 => 'Sony',
-            0x1011 => 'Canon',
-        },
+        RawConv => '$$self{Make} = ($Image::ExifTool::H264::convMake{$val} || "Unknown"); $val',
+        PrintConv => \%convMake,
     },
     # 1 => ModelIDCode according to ref 4/5 (I think not)
     # vals: 0x3001 - Sony HDR-CX105E/TG3E/XR500V
     #       0x1000 - Sony HDR-UX1
     #       0x3000 - Canon HF100 (30p)
     #       0x2000 - Canon HF100 (60i)
+    #       0x3101 - Canon HFM300 (PH, all qualities and frame rates)
+);
+
+# camera info 0xe1 (ref 6)
+%Image::ExifTool::H264::RecInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Camera' },
+    FORMAT => 'int8u',
+    NOTES => 'Recording information stored by some Canon video cameras.',
+    FIRST_ENTRY => 0,
+    0 => {
+        Name => 'RecordingMode',
+        PrintConv => {
+            0x02 => 'XP+', # High Quality 12 Mbps
+            0x04 => 'SP',  # Standard Play 7 Mbps
+            0x05 => 'LP',  # Long Play 5 Mbps
+            0x06 => 'FXP', # High Quality 17 Mbps
+            0x07 => 'MXP', # High Quality 24 Mbps
+        },
+    },
+);
+
+# camera info 0xee (ref 6)
+%Image::ExifTool::H264::FrameInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Video' },
+    FORMAT => 'int8u',
+    NOTES => 'Frame rate information stored by some Canon video cameras.',
+    FIRST_ENTRY => 0,
+    0 => 'CaptureFrameRate',
+    1 => 'VideoFrameRate',
+    # 2 - 8=60i, 10=PF30, 74=PF24 (PH, HFM300)
 );
 
 #==============================================================================
@@ -806,7 +934,13 @@ sub ProcessSEI($$)
         $lastTag = $tag;
         my $buff = substr($$dataPt, $pos + 1, 4);
         my $from;
-        if ($$tagTablePtr{$tag}) {
+        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        if ($tagInfo) {
+            # use our own print conversion for Unknown tags
+            if ($$tagInfo{Unknown} and not $$tagInfo{SetPrintConv}) {
+                $$tagInfo{PrintConv} = 'sprintf("0x%.8x", unpack("N", $val))';
+                $$tagInfo{SetPrintConv} = 1;
+            }
             # combine with next value(s) if necessary
             my $combine = $$tagTablePtr{$tag}{Combine};
             while ($combine) {
@@ -820,20 +954,13 @@ sub ProcessSEI($$)
                 ++$lastTag;
                 --$combine;
             }
-        } else {
-            my %tagInfo = (
-                Name        => sprintf('MDPM_0x%.2x', $tag),
-                Description => sprintf('MDPM 0x%.2x', $tag),
-                Unknown     => 1,
-                PrintConv   => 'sprintf("0x%.8x", unpack("N", $val))',
+            $exifTool->HandleTag($tagTablePtr, $tag, undef,
+                TagInfo => $tagInfo,
+                DataPt  => \$buff,
+                Size    => length($buff),
+                Index   => defined $from ? "$from-$index" : $index,
             );
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, \%tagInfo);
         }
-        $exifTool->HandleTag($tagTablePtr, $tag, undef,
-            DataPt => \$buff,
-            Size   => length($buff),
-            Index  => defined $from ? "$from-$index" : $index,
-        );
         $pos += 5;
     }
     $$exifTool{INDENT} = $oldIndent;
@@ -843,6 +970,7 @@ sub ProcessSEI($$)
 #------------------------------------------------------------------------------
 # Extract information from H.264 video stream
 # Inputs: 0) ExifTool ref, 1) data ref
+# Returns: 0 = done parsing, 1 = we want to parse more of these
 sub ParseH264Video($$)
 {
     my ($exifTool, $dataPt) = @_;
@@ -850,6 +978,7 @@ sub ParseH264Video($$)
     my $out = $exifTool->Options('TextOut');
     my $tagTablePtr = GetTagTable('Image::ExifTool::H264::Main');
     my %parseNalUnit = ( 0x06 => 1, 0x07 => 1 );    # NAL unit types to parse
+    my $foundUserData;
     my $len = length $$dataPt;
     my $pos = 0;
     while ($pos < $len) {
@@ -896,9 +1025,10 @@ sub ParseH264Video($$)
                 next unless $exifTool->Options('ExtractEmbedded');
                 $$exifTool{DOC_NUM} = $$exifTool{GotNAL06};
             }
-            my $result = ProcessSEI($exifTool, { DataPt => \$buff } );
+            $foundUserData = ProcessSEI($exifTool, { DataPt => \$buff } );
             delete $$exifTool{DOC_NUM};
-            next unless $result;
+            # keep parsing SEI's until we find the user data
+            next unless $foundUserData;
             $$exifTool{GotNAL06} = ($$exifTool{GotNAL06} || 0) + 1;
 
         } elsif ($nal_unit_type == 0x07) {  # sequence_parameter_set_rbsp
@@ -911,6 +1041,11 @@ sub ParseH264Video($$)
         # we were successful, so don't parse this NAL unit type again
         delete $parseNalUnit{$nal_unit_type};
     }
+    # parse one extra H264 frame if we didn't find the user data in this one
+    # (Panasonic cameras don't put the SEI in the first frame)
+    return 0 if $foundUserData or $$exifTool{ParsedH264};
+    $$exifTool{ParsedH264} = 1;
+    return 1;
 }
 
 1;  # end
@@ -932,7 +1067,7 @@ information from H.264 video streams.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
