@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.26';
+$VERSION = '1.31';
 
 sub ProcessID3v2($$$);
 sub ProcessPrivate($$$);
@@ -25,10 +25,10 @@ sub ConvertID3v1Text($$);
 # audio formats that we process after an ID3v2 header (in order)
 my @audioFormats = qw(APE MPC FLAC OGG MP3);
 
-# audio formats where the processing proc is in a different module
+# audio formats where the processing proc is in a differently-named module
 my %audioModule = (
     MP3 => 'ID3',
-    OGG => 'Vorbis',
+    OGG => 'Ogg',
 );
 
 # picture types for 'PIC' and 'APIC' tags
@@ -62,8 +62,6 @@ my %dateTimeConv = (
     PrintConv => '$self->ConvertDateTime($val)',
 );
 
-my %warnedOnce;     # hash of warnings we issued
-
 # This table is just for documentation purposes
 %Image::ExifTool::ID3::Main = (
     VARS => { NO_ID => 1 },
@@ -392,7 +390,7 @@ my %genre = (
         Notes => 'uses same lookup table as ID3v1 Genre',
         PrintConv => 'Image::ExifTool::ID3::PrintGenre($val)',
     },
-    TCP => 'Compilation', # not part of spec, but used by iTunes
+    TCP => { Name => 'Compilation', PrintConv => { 0 => 'No', 1 => 'Yes' } }, # iTunes
     TCR => { Name => 'Copyright', Groups => { 2 => 'Author' } },
     TDA => { Name => 'Date', Groups => { 2 => 'Time' } },
     TDY => 'PlaylistDelay',
@@ -433,6 +431,15 @@ my %genre = (
     WCP => { Name => 'CopyrightURL', Groups => { 2 => 'Author' } },
     WPB => 'PublisherURL',
     WXX => 'UserDefinedURL',
+    # the following written by iTunes 10.5 (ref PH)
+    RVA => 'RelativeVolumeAdjustment',
+    TST => 'TitleSortOrder',
+    TSA => 'AlbumSortOrder',
+    TSP => 'PerformerSortOrder',
+    TS2 => 'AlbumArtistSortOrder',
+    TSC => 'ComposerSortOrder',
+    ITU => { Name => 'iTunesU', Description => 'iTunes U', Binary => 1, Unknown => 1 },
+    PCS => { Name => 'Podcast', Binary => 1, Unknown => 1 },
 );
 
 # tags common to ID3v2.3 and ID3v2.4
@@ -518,7 +525,7 @@ my %id3v2_common = (
     TSRC => 'ISRC', # (international standard recording code)
     TSSE => 'EncoderSettings',
     TXXX => 'UserDefinedText',
-  # UFID => 'UniqueFileID',
+  # UFID => 'UniqueFileID', (not extracted because it is long and nasty and not very useful)
     USER => 'TermsOfUse',
     USLT => 'Lyrics',
     WCOM => 'CommercialURL',
@@ -556,7 +563,7 @@ my %id3v2_common = (
     NOTES => 'ID3 version 2.4 tags',
     %id3v2_common,  # include common tags
   # EQU2 => 'Equalization',
-  # RVA2 => 'RelativeVolumeAdjustment',
+    RVA2 => 'RelativeVolumeAdjustment',
   # SEEK => 'Seek',
   # SIGN => 'Signature',
     TDEN => { Name => 'EncodingTime',       Groups => { 2 => 'Time' }, %dateTimeConv },
@@ -572,6 +579,11 @@ my %id3v2_common = (
     TSOP => 'PerformerSortOrder',
     TSOT => 'TitleSortOrder',
     TSST => 'SetSubtitle',
+    # the following are written by iTunes 10.5 (ref PH)
+    TSO2 => 'AlbumArtistSortOrder',
+    TSOC => 'ComposerSortOrder',
+    ITNU => { Name => 'iTunesU', Description => 'iTunes U', Binary => 1, Unknown => 1 },
+    PCST => { Name => 'Podcast', Binary => 1, Unknown => 1 },
 );
 
 # ID3 PRIV tags (ref PH)
@@ -691,11 +703,11 @@ sub PrintGenre($)
     }
     # (genre numbers are separated by nulls in ID3v2.4,
     #  but nulls are converted to '/' by DecodeString())
-    while ($val =~ /(?:^|\/)(\d+)/g) {
+    while ($val =~ /(?:^|\/)(\d+)(\/|$)/g) {
         $genre{$1} or $genre{$1} = "Unknown ($1)";
     }
     $val =~ s/\((\d+)\)/\($genre{$1}\)/g;
-    $val =~ s/(^|\/)(\d+)/$1$genre{$2}/g;
+    $val =~ s/(^|\/)(\d+)(\/|$)/$1$genre{$2}$3/g;
     $val =~ s/^\(([^)]+)\)\1?$/$1/; # clean up by removing brackets and duplicates
     return $val;
 }
@@ -746,18 +758,6 @@ sub DecodeString($$;$)
     return join('/',@vals);
 }
 
-#------------------------------------------------------------------------------
-# Issue one warning of each type
-# Inputs: 0) ExifTool object reference, 1) warning
-sub WarnOnce($$)
-{
-    my ($exifTool, $warn) = @_;
-    unless ($warnedOnce{$warn}) {
-        $warnedOnce{$warn} = 1;
-        $exifTool->Warn($warn);
-    }
-}
-
 #------------------------------------------------------------------------------
 # Convert sync-safe integer to a number we can use
 # Inputs: 0) int32u sync-safe value
@@ -786,7 +786,6 @@ sub ProcessID3v2($$$)
     my $len;    # frame data length
 
     $verbose and $exifTool->VerboseDir($tagTablePtr->{GROUPS}->{1}, 0, $size);
-    undef %warnedOnce;
 
     for (;;$offset+=$len) {
         my ($id, $flags, $hi);
@@ -852,7 +851,7 @@ sub ProcessID3v2($$$)
             }
         }
         if ($flags{Encrypt}) {
-            WarnOnce($exifTool, 'Encrypted frames currently not supported');
+            $exifTool->WarnOnce('Encrypted frames currently not supported');
             next;
         }
         # extract the value
@@ -886,7 +885,7 @@ sub ProcessID3v2($$$)
                     next;
                 }
             } else {
-                WarnOnce($exifTool,'Install Compress::Zlib to decode compressed frames');
+                $exifTool->WarnOnce('Install Compress::Zlib to decode compressed frames');
                 next;
             }
         }
@@ -942,7 +941,7 @@ sub ProcessID3v2($$$)
             foreach (0..1) { $vals[$_] = '' unless defined $vals[$_]; }
             $val = length($vals[0]) ? "($vals[0]) $vals[1]" : $vals[1];
         } elsif ($id eq 'USER') {
-            $valLen > 4 or $exifTool->Warn('Short USER frame'), next;
+            $valLen > 4 or $exifTool->Warn("Short $id frame"), next;
             $lang = substr($val,1,3);
             $val = DecodeString($exifTool, substr($val,4), Get8u(\$val,0));
         } elsif ($id =~ /^(CNT|PCNT)$/) {
@@ -972,21 +971,70 @@ sub ProcessID3v2($$$)
                 $exifTool->HandleTag($tagTablePtr, "$id-$i", $attr);
                 ++$i;
             }
+        } elsif ($id eq 'RVA' or $id eq 'RVAD') {
+            my @dat = unpack('C*', $val);
+            my $flag = shift @dat;
+            my $bits = shift @dat or $exifTool->Warn("Short $id frame"), next;
+            my $bytes = int(($bits + 7) / 8);
+            my @parse = (['Right',0,2,0x01],['Left',1,3,0x02],['Back-right',4,6,0x04],
+                         ['Back-left',5,7,0x08],['Center',8,9,0x10],['Bass',10,11,0x20]);
+            $val = '';
+            while (@parse) {
+                my $elem = shift @parse;
+                my $j = $$elem[2] * $bytes;
+                last if scalar(@dat) < $j + $bytes;
+                my $i = $$elem[1] * $bytes;
+                $val .= ', ' if $val;
+                my ($rel, $pk, $b);
+                for ($rel=0, $pk=0, $b=0; $b<$bytes; ++$b) {
+                    $rel = $rel * 256 + $dat[$i + $b];
+                    $pk  = $pk  * 256 + $dat[$j + $b]; # (peak - not used in printout)
+                }
+                $rel =-$rel unless $flag & $$elem[3];
+                $val .= sprintf("%+.1f%% %s", 100 * $rel / ((1<<$bits)-1), $$elem[0]);
+            }
+        } elsif ($id eq 'RVA2') {
+            my ($pos, $id) = $val=~/^([^\0]*)\0/ ? (length($1)+1, $1) : (1, '');
+            my @vals;
+            while ($pos + 4 <= $valLen) {
+                my $type = Get8u(\$val, $pos);
+                my $str = ({
+                    0 => 'Other',
+                    1 => 'Master',
+                    2 => 'Front-right',
+                    3 => 'Front-left',
+                    4 => 'Back-right',
+                    5 => 'Back-left',
+                    6 => 'Front-centre',
+                    7 => 'Back-centre',
+                    8 => 'Subwoofer',
+                }->{$type} || "Unknown($type)");
+                my $db = Get16s(\$val,$pos+1) / 512;
+                # convert dB to percent as displayed by iTunes 10.5
+                # (not sure why I need to divide by 20 instead of 10 as expected - PH)
+                push @vals, sprintf('%+.1f%% %s', 10**($db/20+2)-100, $str);
+                # step to next channel (ignoring peak volume)
+                $pos += 4 + int((Get8u(\$val,$pos+3) + 7) / 8);
+            }
+            $val = join ', ', @vals;
+            $val .= " ($id)" if $id;
         } elsif ($id eq 'PRIV') {
             # save version number to set group1 name for tag later
             $exifTool->{ID3_Ver} = $tagTablePtr->{GROUPS}->{1};
             $exifTool->HandleTag($tagTablePtr, $id, $val);
             next;
+        } elsif ($$tagInfo{Format}) {
+            $exifTool->HandleTag($tagTablePtr, $id, undef, DataPt => \$val);
+            next;
         } elsif (not $$tagInfo{Binary}) {
             $exifTool->Warn("Don't know how to handle $id frame");
             next;
         }
-        if ($lang and $lang =~ /^[a-z]{3}$/i) {
+        if ($lang and $lang =~ /^[a-z]{3}$/i and $lang ne 'eng') {
             $tagInfo = Image::ExifTool::GetLangInfo($tagInfo, lc $lang);
         }
         $exifTool->FoundTag($tagInfo, $val);
     }
-    undef %warnedOnce;
 }
 
 #------------------------------------------------------------------------------
@@ -1201,7 +1249,7 @@ other types of audio files.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
