@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.15';
+$VERSION = '1.18';
 
 sub ProcessJpeg2000Box($$$);
 
@@ -63,7 +63,42 @@ my %uuid = (
   # 'UUID-GeoJP2' => "\xb1\x4b\xf8\xbd\x08\x3d\x4b\x43\xa5\xae\x8c\xd7\xd5\xa6\xce\x03",
 );
 
+# JPEG2000 codestream markers (ref ISO/IEC FCD15444-1/2)
+my %j2cMarker = (
+    0x4f => 'SOC', # start of codestream
+    0x51 => 'SIZ', # image and tile size
+    0x52 => 'COD', # coding style default
+    0x53 => 'COC', # coding style component
+    0x55 => 'TLM', # tile-part lengths
+    0x57 => 'PLM', # packet length, main header
+    0x58 => 'PLT', # packet length, tile-part header
+    0x5c => 'QCD', # quantization default
+    0x5d => 'QCC', # quantization component
+    0x5e => 'RGN', # region of interest
+    0x5f => 'POD', # progression order default
+    0x60 => 'PPM', # packed packet headers, main
+    0x61 => 'PPT', # packed packet headers, tile-part
+    0x63 => 'CRG', # component registration
+    0x64 => 'CME', # comment and extension
+    0x90 => 'SOT', # start of tile-part
+    0x91 => 'SOP', # start of packet
+    0x92 => 'EPH', # end of packet header
+    0x93 => 'SOD', # start of data
+    # extensions (ref ISO/IEC FCD15444-2)
+    0x70 => 'DCO', # variable DC offset
+    0x71 => 'VMS', # visual masking
+    0x72 => 'DFS', # downsampling factor style
+    0x73 => 'ADS', # arbitrary decomposition style
+  # 0x72 => 'ATK', # arbitrary transformation kernels ?
+    0x78 => 'CBD', # component bit depth
+    0x74 => 'MCT', # multiple component transformation definition
+    0x75 => 'MCC', # multiple component collection
+    0x77 => 'MIC', # multiple component intermediate collection
+    0x76 => 'NLT', # non-linearity point transformation
+);
+
 # JPEG 2000 "box" (ie. atom) names
+# Note: only tags with a defined "Format" are extracted
 %Image::ExifTool::Jpeg2000::Main = (
     GROUPS => { 2 => 'Image' },
     PROCESS_PROC => \&ProcessJpeg2000Box,
@@ -92,31 +127,12 @@ my %uuid = (
             },
         },
         bpcc => 'BitsPerComponent',
-        colr => [
-            {
-                Name => 'ICC_Profile',
-                Condition => '$$valPt =~ /^(\x02|\x03)/',
-                SubDirectory => {
-                    TagTable => 'Image::ExifTool::ICC_Profile::Main',
-                    Start => '$valuePtr + 3',
-                },
+        colr => {
+            Name => 'ColorSpecification',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Jpeg2000::ColorSpec',
             },
-            {
-                Name => 'Colorspace',
-                Condition => '$$valPt =~ /^\x01/',
-                Format => 'binary',
-                ValueConv => 'unpack("x3N", $val)',
-                PrintConv => {
-                    16 => 'sRGB',
-                    17 => 'Grayscale',
-                    18 => 'sYCC',
-                },
-            },
-            {
-                Name => 'ColorSpecification',
-                Binary => 1,
-            },
-        ],
+        },
         pclr => 'Palette',
         cdef => 'ComponentDefinition',
        'res '=> {
@@ -192,6 +208,7 @@ my %uuid = (
     uuid => [
         {
             Name => 'UUID-EXIF',
+            # (this is the EXIF that we create)
             Condition => '$$valPt=~/^JpgTiffExif->JP2/',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Exif::Main',
@@ -201,14 +218,36 @@ my %uuid = (
                 Start => '$valuePtr + 16',
             },
         },
+        {
+            Name => 'UUID-EXIF2',
+            # written by Photoshop 7.01+Adobe JPEG2000-plugin v1.5
+            Condition => '$$valPt=~/^\x05\x37\xcd\xab\x9d\x0c\x44\x31\xa7\x2a\xfa\x56\x1f\x2a\x11\x3e/',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Exif::Main',
+                ProcessProc => \&Image::ExifTool::ProcessTIFF,
+                WriteProc => \&Image::ExifTool::WriteTIFF,
+                DirName => 'EXIF',
+                Start => '$valuePtr + 16',
+            },
+        },
         {
             Name => 'UUID-IPTC',
+            # (this is the IPTC that we create)
             Condition => '$$valPt=~/^\x33\xc7\xa4\xd2\xb8\x1d\x47\x23\xa0\xba\xf1\xa3\xe0\x97\xad\x38/',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::IPTC::Main',
                 Start => '$valuePtr + 16',
             },
         },
+        {
+            Name => 'UUID-IPTC2',
+            # written by Photoshop 7.01+Adobe JPEG2000-plugin v1.5
+            Condition => '$$valPt=~/^\x09\xa1\x4e\x97\xc0\xb4\x42\xe0\xbe\xbf\x36\xdf\x6f\x0c\xe3\x6f/',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::IPTC::Main',
+                Start => '$valuePtr + 16',
+            },
+        },
         {
             Name => 'UUID-XMP',
             # ref http://www.adobe.com/products/xmp/pdfs/xmpspec.pdf
@@ -228,9 +267,23 @@ my %uuid = (
                 Start => '$valuePtr + 16',
             },
         },
+        {
+            Name => 'UUID-Photoshop',
+            # written by Photoshop 7.01+Adobe JPEG2000-plugin v1.5
+            Condition => '$$valPt=~/^\x2c\x4c\x01\x00\x85\x04\x40\xb9\xa0\x3e\x56\x21\x48\xd6\xdf\xeb/',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Photoshop::Main',
+                Start => '$valuePtr + 16',
+            },
+        },
         {
             Name => 'UUID-Unknown',
         },
+        # also written by Adobe JPEG2000 plugin v1.5:
+        # 3a 0d 02 18 0a e9 41 15 b3 76 4b ca 41 ce 0e 71 - 1 byte (01)
+        # 47 c9 2c cc d1 a1 45 81 b9 04 38 bb 54 67 71 3b - 1 byte (01)
+        # bc 45 a7 74 dd 50 4e c6 a9 f6 f3 a1 37 f4 7e 90 - 4 bytes (00 00 00 32)
+        # d7 c8 c5 ef 95 1f 43 b2 87 57 04 25 00 f5 38 e8 - 4 bytes (00 00 00 32)
     ],
     uinf => {
         Name => 'UUIDInfo',
@@ -359,6 +412,81 @@ my %uuid = (
     },
 );
 
+%Image::ExifTool::Jpeg2000::ColorSpec = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Image' },
+    FORMAT => 'int8s',
+    0 => {
+        Name => 'ColorSpecMethod',
+        RawConv => '$$self{ColorSpecMethod} = $val',
+        PrintConv => {
+            1 => 'Enumerated',
+            2 => 'Restricted ICC',
+            3 => 'Any ICC',
+            4 => 'Vendor Color',
+        },
+    },
+    1 => 'ColorSpecPrecedence',
+    2 => {
+        Name => 'ColorSpecApproximation',
+        PrintConv => {
+            0 => 'Not Specified',
+            1 => 'Accurate',
+            2 => 'Exceptional Quality',
+            3 => 'Reasonable Quality',
+            4 => 'Poor Quality',
+        },
+    },
+    3 => [
+        {
+            Name => 'ICC_Profile',
+            Condition => q{
+                $$self{ColorSpecMethod} == 2 or
+                $$self{ColorSpecMethod} == 3
+            },
+            Format => 'undef[$size-3]',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::ICC_Profile::Main',
+            },
+        },
+        {
+            Name => 'ColorSpace',
+            Condition => '$$self{ColorSpecMethod} == 1',
+            Format => 'int32u',
+            PrintConv => { # ref 15444-2 2002-05-15
+                0 => 'Bi-level',
+                1 => 'YCbCr(1)',
+                3 => 'YCbCr(2)',
+                4 => 'YCbCr(3)',
+                9 => 'PhotoYCC',
+                11 => 'CMY',
+                12 => 'CMYK',
+                13 => 'YCCK',
+                14 => 'CIELab',
+                15 => 'Bi-level(2)', # (incorrectly listed as 18 in 15444-2 2000-12-07)
+                16 => 'sRGB',
+                17 => 'Grayscale',
+                18 => 'sYCC',
+                19 => 'CIEJab',
+                20 => 'e-sRGB',
+                21 => 'ROMM-RGB',
+                # incorrect in 15444-2 2000-12-07
+                #22 => 'sRGB based YCbCr',
+                #23 => 'YPbPr(1125/60)',
+                #24 => 'YPbPr(1250/50)',
+                22 => 'YPbPr(1125/60)',
+                23 => 'YPbPr(1250/50)',
+                24 => 'e-sYCC',
+            },
+        },
+        {
+            Name => 'ColorSpecData',
+            Format => 'undef[$size-3]',
+            Binary => 1,
+        },
+    ],
+);
+
 #------------------------------------------------------------------------------
 # Create new JPEG 2000 boxes when writing
 # (Currently only supports adding certain UUID boxes)
@@ -381,6 +509,9 @@ sub CreateNewBoxes($$)
                 DirName => $$subdir{DirName} || $dirName,
                 Parent => 'JP2',
             );
+            # remove "UUID-" from start of directory name to allow appropriate
+            # directories to be written as a block
+            $dirInfo{DirName} =~ s/^UUID-//;
             my $newdir = $exifTool->WriteDirectory(\%dirInfo, $tagTable, $$subdir{WriteProc});
             if (defined $newdir and length $newdir) {
                 my $boxhdr = pack('N', length($newdir) + 24) . 'uuid' . $uuid{$dirName};
@@ -405,6 +536,7 @@ sub ProcessJpeg2000Box($$$)
     my $dataPos = $$dirInfo{DataPos};
     my $dirLen = $$dirInfo{DirLen} || 0;
     my $dirStart = $$dirInfo{DirStart} || 0;
+    my $base = $$dirInfo{Base} || 0;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
     my $dirEnd = $dirStart + $dirLen;
@@ -419,13 +551,14 @@ sub ProcessJpeg2000Box($$$)
     } else {
         # (must not set verbose flag when writing!)
         $verbose = $exifTool->{OPTIONS}->{Verbose};
+        $exifTool->VerboseDir($$dirInfo{DirName}) if $verbose;
     }
     # loop through all contained boxes
     my ($pos, $boxLen);
     for ($pos=$dirStart; ; $pos+=$boxLen) {
         my ($boxID, $buff, $valuePtr);
         if ($raf) {
-            $dataPos = $raf->Tell();
+            $dataPos = $raf->Tell() - $base;
             my $n = $raf->Read($buff,8);
             unless ($n == 8) {
                 $n and $err = '', last;
@@ -460,6 +593,9 @@ sub ProcessJpeg2000Box($$$)
                     while ($raf->Read($buff, 65536)) {
                         Write($outfile, $buff) or $err = 1;
                     }
+                } elsif ($verbose) {
+                    my $msg = sprintf("offset 0x%.4x to end of file", $dataPos + $base + $pos);
+                    $exifTool->VPrint(0, "$$exifTool{INDENT}- Tag '$boxID' ($msg)\n");
                 }
                 last;   # (ignore the rest of the file when reading)
             }
@@ -486,7 +622,7 @@ sub ProcessJpeg2000Box($$$)
         }
         if ($raf) {
             # read the box data
-            $dataPos = $raf->Tell();
+            $dataPos = $raf->Tell() - $base;
             $raf->Read($buff,$boxLen) == $boxLen or $err = '', last;
             $valuePtr = 0;
             $dataLen = $boxLen;
@@ -498,7 +634,7 @@ sub ProcessJpeg2000Box($$$)
         }
         if (defined $tagInfo and not $tagInfo) {
             # GetTagInfo() required the value for a Condition
-            my $tmpVal = substr($$dataPt, $valuePtr, $boxLen < 48 ? $boxLen : 48);
+            my $tmpVal = substr($$dataPt, $valuePtr, $boxLen < 128 ? $boxLen : 128);
             $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $boxID, \$tmpVal);
         }
         # delete all UUID boxes if deleting all information
@@ -513,6 +649,7 @@ sub ProcessJpeg2000Box($$$)
                 DataPt => $dataPt,
                 Size   => $boxLen,
                 Start  => $valuePtr,
+                Addr   => $valuePtr + $dataPos + $base,
             );
             next unless $tagInfo;
         }
@@ -527,14 +664,16 @@ sub ProcessJpeg2000Box($$$)
             my %subdirInfo = (
                 Parent => 'JP2',
                 DataPt => $dataPt,
-                DataPos => $dataPos,
+                DataPos => -$subdirStart, # (relative to Base)
                 DataLen => $dataLen,
                 DirStart => $subdirStart,
                 DirLen => $subdirLen,
                 DirName => $$subdir{DirName} || $$tagInfo{Name},
                 OutFile => $outfile,
-                Base => $dataPos + $subdirStart,
+                Base => $base + $dataPos + $subdirStart,
             );
+            # remove "UUID-" prefix to allow appropriate directories to be written as a block
+            $subdirInfo{DirName} =~ s/^UUID-//;
             my $subTable = GetTagTable($$subdir{TagTable}) || $tagTablePtr;
             if ($outfile) {
                 # remove this directory from our create list
@@ -586,17 +725,30 @@ sub ProcessJpeg2000Box($$$)
 # Returns: 1 on success, 0 if this wasn't a valid JPEG 2000 file, or -1 on write error
 sub ProcessJP2($$)
 {
+    local $_;
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
-    my $rtnVal = 0;
     my $hdr;
 
     # check to be sure this is a valid JPG2000 file
     return 0 unless $raf->Read($hdr,12) == 12;
-    return 0 unless $hdr eq "\x00\x00\x00\x0cjP  \x0d\x0a\x87\x0a" or     # (ref 1)
-                    $hdr eq "\x00\x00\x00\x0cjP\x1a\x1a\x0d\x0a\x87\x0a"; # (ref 2)
-
+    unless ($hdr eq "\x00\x00\x00\x0cjP  \x0d\x0a\x87\x0a" or     # (ref 1)
+            $hdr eq "\x00\x00\x00\x0cjP\x1a\x1a\x0d\x0a\x87\x0a") # (ref 2)
+    {
+        return 0 unless $hdr =~ /^\xff\x4f\xff\x51\0/;  # check for JP2 codestream format
+        if ($outfile) {
+            $exifTool->Error('Writing of J2C files is not yet supported');
+            return 0
+        }
+        # add J2C markers if not done already
+        unless ($Image::ExifTool::jpegMarker{0x4f}) {
+            $Image::ExifTool::jpegMarker{$_} = $j2cMarker{$_} foreach keys %j2cMarker;
+        }
+        $exifTool->SetFileType('J2C');
+        $raf->Seek(0,0);
+        return $exifTool->ProcessJPEG($dirInfo);    # decode with JPEG processor
+    }
     if ($outfile) {
         Write($outfile, $hdr) or return -1;
         $exifTool->InitWriteDirs(\%jp2Map);
@@ -642,7 +794,7 @@ files.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
