@@ -505,7 +505,7 @@ sub ProcessLinkInfo($$$)
         # read Volume ID
         $off = Get32u($dataPt, 0x0c);
         if ($off + 0x20 <= $dataLen) {
-            my $len = Get32u($dataPt, $off);
+            # my $len = Get32u($dataPt, $off);
             $exifTool->HandleTag($tagTablePtr, 'DriveType', undef, %opts, Start=>$off+4);
             $pos = Get32u($dataPt, $off + 0x0c);
             if ($pos == 0x14) {
@@ -698,7 +698,7 @@ information MS Shell Link (Windows shortcut) files.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
