@@ -9,6 +9,7 @@
 #               2) http://www.getid3.org/
 #               3) http://dvd.sourceforge.net/dvdinfo/dvdmpeg.html
 #               4) http://ffmpeg.org/
+#               5) http://sourceforge.net/projects/mediainfo/
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::MPEG;
@@ -17,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.11';
+$VERSION = '1.13';
 
 %Image::ExifTool::MPEG::Audio = (
     GROUPS => { 2 => 'Audio' },
@@ -46,7 +47,7 @@ $VERSION = '1.11';
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 3',
             Notes => 'version 1, layer 1',
             PrintConvColumns => 3,
-            PrintConv => {
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 64000,
@@ -63,13 +64,14 @@ $VERSION = '1.11';
                 13 => 416000,
                 14 => 448000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 2',
             Notes => 'version 1, layer 2',
             PrintConvColumns => 3,
-            PrintConv => {
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 48000,
@@ -86,13 +88,14 @@ $VERSION = '1.11';
                 13 => 320000,
                 14 => 384000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} == 3 and $self->{MPEG_Layer} == 1',
             Notes => 'version 1, layer 3',
             PrintConvColumns => 3,
-            PrintConv => {
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 40000,
@@ -109,13 +112,14 @@ $VERSION = '1.11';
                 13 => 256000,
                 14 => 320000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} != 3 and $self->{MPEG_Layer} == 3',
             Notes => 'version 2 or 2.5, layer 1',
             PrintConvColumns => 3,
-            PrintConv => {
+            ValueConv => {
                 0 => 'free',
                 1 => 32000,
                 2 => 48000,
@@ -132,13 +136,14 @@ $VERSION = '1.11';
                 13 => 224000,
                 14 => 256000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
         {
             Name => 'AudioBitrate',
             Condition => '$self->{MPEG_Vers} != 3 and $self->{MPEG_Layer}',
             Notes => 'version 2 or 2.5, layer 2 or 3',
             PrintConvColumns => 3,
-            PrintConv => {
+            ValueConv => {
                 0 => 'free',
                 1 => 8000,
                 2 => 16000,
@@ -155,6 +160,7 @@ $VERSION = '1.11';
                 13 => 144000,
                 14 => 160000,
             },
+            PrintConv => 'ConvertBitrate($val)',
         },
     ],
     'Bit20-21' => [
@@ -305,6 +311,7 @@ $VERSION = '1.11';
     'Bit32-49' => {
         Name => 'VideoBitrate',
         ValueConv => '$val eq 0x3ffff ? "Variable" : $val * 400',
+        PrintConv => 'ConvertBitrate($val)',
     },
     # these tags not very interesting
     #'Bit50'    => 'MarkerBit',
@@ -313,13 +320,66 @@ $VERSION = '1.11';
     #'Bit62'    => 'IntraQuantMatrixFlag',
 );
 
-%Image::ExifTool::MPEG::VBR = (
+%Image::ExifTool::MPEG::Xing = (
     GROUPS => { 2 => 'Audio' },
     VARS => { NO_ID => 1 },
-    NOTES => 'These tags are extracted for variable bitrate audio.',
+    NOTES => 'These tags are extracted from the Xing/Info frame.',
     1 => { Name => 'VBRFrames' },
     2 => { Name => 'VBRBytes' },
     3 => { Name => 'VBRScale' },
+    4 => { Name => 'Encoder' },
+    5 => { Name => 'LameVBRQuality' },
+    6 => { Name => 'LameQuality' },
+    7 => { # (for documentation only)
+        Name => 'LameHeader',
+        SubDirectory => { TagTable => 'Image::ExifTool::MPEG::Lame' },
+    },
+);
+
+# Lame header tags (ref 5)
+%Image::ExifTool::MPEG::Lame = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    NOTES => 'Tags extracted from Lame 3.90 or later header.',
+    9 => {
+        Name => 'LameMethod',
+        Mask => 0x0f,
+        PrintConv => {
+            1 => 'CBR',
+            2 => 'ABR',
+            3 => 'VBR (old/rh)',
+            4 => 'VBR (new/mtrh)',
+            5 => 'VBR (old/rh)',
+            6 => 'VBR',
+            8 => 'CBR (2-pass)',
+            9 => 'ABR (2-pass)',
+        },
+    },
+    10 => {
+        Name => 'LameLowPassFilter',
+        ValueConv => '$val * 100',
+        PrintConv => '($val / 1000) . " kHz"',
+    },
+    # 19 - EncodingFlags
+    20 => {
+        Name => 'LameBitrate',
+        ValueConv => '$val * 1000',
+        PrintConv => 'ConvertBitrate($val)',
+    },
+    24 => {
+        Name => 'LameStereoMode',
+        Mask => 0x1c,
+        ValueConv => '$val >> 2',
+        PrintConv => {
+            0 => 'Mono',
+            1 => 'Stereo',
+            2 => 'Dual Channels',
+            3 => 'Joint Stereo',
+            4 => 'Forced Joint Stereo',
+            6 => 'Auto',
+            7 => 'Intensity Stereo',
+        },
+    },
 );
 
 # composite tags
@@ -348,12 +408,28 @@ $VERSION = '1.11';
             # calculate duration as file size divided by total bitrate
             # (note: this is only approximate!)
             return undef unless $val[2] or $val[3];
-            return undef if $prt[2] and not $prt[2] =~ /^\d+$/;
+            return undef if $val[2] and not $val[2] =~ /^\d+$/;
             return undef if $val[3] and not $val[3] =~ /^\d+$/;
-            return (8 * ($val[0] - ($val[1]||0))) / (($prt[2]||0) + ($val[3]||0));
+            return (8 * ($val[0] - ($val[1]||0))) / (($val[2]||0) + ($val[3]||0));
         },
         PrintConv => 'ConvertDuration($val) . " (approx)"',
     },
+    AudioBitrate => {
+        Groups => { 2 => 'Audio' },
+        Notes => 'calculated for variable-bitrate MPEG audio',
+        Require => {
+            0 => 'MPEG:MPEGAudioVersion',
+            1 => 'MPEG:SampleRate',
+            2 => 'MPEG:VBRBytes',
+            3 => 'MPEG:VBRFrames',
+        },
+        ValueConv => q{
+            return undef unless $val[3];
+            my $mfs = $prt[1] / ($val[0] == 3 ? 144 : 72);
+            return $mfs * $val[2] / $val[3];
+        },
+        PrintConv => 'ConvertBitrate($val)',
+    },
 );
 
 # add our composite tags
@@ -429,17 +505,19 @@ sub ParseMPEGAudio($$)
         last if $pos + 8 > $len;
         my $buff = substr($$buffPt, $pos, 8);
         last unless $buff =~ /^(Xing|Info)/;
-        my $vbrTable = GetTagTable('Image::ExifTool::MPEG::VBR');
+        my $xingTable = GetTagTable('Image::ExifTool::MPEG::Xing');
+        my $vbrScale;
         my $flags = unpack('x4N', $buff);
+        my $isVBR = ($buff !~ /^Info/);     # Info frame is not VBR (ref 5)
         $pos += 8;
         if ($flags & 0x01) {    # VBRFrames
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 1, unpack("x${pos}N", $$buffPt));
+            $exifTool->HandleTag($xingTable, 1, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x02) {    # VBRBytes
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 2, unpack("x${pos}N", $$buffPt));
+            $exifTool->HandleTag($xingTable, 2, unpack("x${pos}N", $$buffPt)) if $isVBR;
             $pos += 4;
         }
         if ($flags & 0x04) {    # VBR_TOC
@@ -449,10 +527,53 @@ sub ParseMPEGAudio($$)
         }
         if ($flags & 0x08) {    # VBRScale
             last if $pos + 4 > $len;
-            $exifTool->HandleTag($vbrTable, 3, unpack("x${pos}N", $$buffPt));
+            $vbrScale = unpack("x${pos}N", $$buffPt);
+            $exifTool->HandleTag($xingTable, 3, $vbrScale) if $isVBR;
             $pos += 4;
         }
-        last;
+        # process Lame header (ref 5)
+        if ($flags & 0x10) {    # Lame
+            last if $pos + 348 > $len;
+        } elsif ($pos + 4 <= $len) {
+            my $lib = substr($$buffPt, $pos, 4);
+            unless ($lib eq 'LAME' or $lib eq 'GOGO') {
+                # attempt to identify other encoders
+                my $n;
+                if (index($$buffPt, 'RCA mp3PRO Encoder') >= 0) {
+                    $lib = 'RCA mp3PRO';
+                } elsif (($n = index($$buffPt, 'THOMSON mp3PRO Encoder')) >= 0) {
+                    $lib = 'Thomson mp3PRO';
+                    $n += 22;
+                    $lib .= ' ' . substr($$buffPt, $n, 6) if length($$buffPt) - $n >= 6;
+                } elsif (index($$buffPt, 'MPGE') >= 0) {
+                    $lib = 'Gogo (<3.0)';
+                } else {
+                    last;
+                }
+                $exifTool->HandleTag($xingTable, 4, $lib);
+                last;
+            }
+        }
+        my $lameLen = $len - $pos;
+        last if $lameLen < 9;
+        my $enc = substr($$buffPt, $pos, 9);
+        if ($enc ge 'LAME3.90') {
+            $exifTool->HandleTag($xingTable, 4, $enc);
+            if ($vbrScale <= 100) {
+                $exifTool->HandleTag($xingTable, 5, int((100 - $vbrScale) / 10));
+                $exifTool->HandleTag($xingTable, 6, (100 - $vbrScale) % 10);
+            }
+            my %dirInfo = (
+                DataPt   => $buffPt,
+                DirStart => $pos,
+                DirLen   => length($$buffPt) - $pos,
+            );
+            my $subTablePtr = GetTagTable('Image::ExifTool::MPEG::Lame');
+            $exifTool->ProcessDirectory(\%dirInfo, $subTablePtr);
+        } else {
+            $exifTool->HandleTag($xingTable, 4, substr($$buffPt, $pos, 20));
+        }
+        last;   # (didn't want to loop anyway)
 	}
 
     return 1;
@@ -582,7 +703,7 @@ based on unofficial sources which may be incomplete, inaccurate or outdated.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -599,6 +720,8 @@ under the same terms as Perl itself.
 
 =item L<http://ffmpeg.org/>
 
+=item L<http://sourceforge.net/projects/mediainfo/>
+
 =back
 
 =head1 SEE ALSO
