@@ -12,6 +12,7 @@
 #               May  18/05 - P. Harvey Tolerate round-off errors in floats.
 #               Feb. 02/08 - P. Harvey Allow different timezones in time values
 #               Sep. 16/08 - P. Harvey Improve timezone testing
+#               Jul. 14/10 - P. Harvey Added writeInfo()
 #------------------------------------------------------------------------------
 
 package t::TestLib;
@@ -22,15 +23,16 @@ require Exporter;
 use Image::ExifTool qw(ImageInfo);
 
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '1.16';
+$VERSION = '1.18';
 @ISA = qw(Exporter);
-@EXPORT = qw(check writeCheck testCompare binaryCompare testVerbose);
+@EXPORT = qw(check writeCheck writeInfo testCompare binaryCompare testVerbose);
 
 sub closeEnough($$);
 sub formatValue($);
+sub writeInfo($$;$$);
 
 #------------------------------------------------------------------------------
-# compare 2 binary files
+# Compare 2 binary files
 # Inputs: 0) file name 1, 1) file name 2
 # Returns: 1 if files are identical
 sub binaryCompare($$)
@@ -57,11 +59,11 @@ sub binaryCompare($$)
 
 #------------------------------------------------------------------------------
 # Compare 2 files and return true and erase the 2nd file if they are the same
-# Inputs: 0) file1, 1) file2
+# Inputs: 0) file1, 1) file2, 2) test number, 3) flag to not erase test file
 # Returns: true if files are the same
-sub testCompare($$$)
+sub testCompare($$$;$)
 {
-    my ($stdfile, $testfile, $testnum) = @_;
+    my ($stdfile, $testfile, $testnum, $keep) = @_;
     my $success = 0;
     my $linenum;
     
@@ -96,8 +98,7 @@ sub testCompare($$$)
                 warn "\n  Test $testnum differs beginning at line $linenum:\n";
                 defined $line1 or $line1 = '(null)';
                 defined $line2 or $line2 = '(null)';
-                chomp $line1;
-                chomp $line2;
+                chomp($line1,$line2);
                 warn qq{    Test gave: "$line2"\n};
                 warn qq{    Should be: "$line1"\n};
             }
@@ -108,7 +109,7 @@ sub testCompare($$$)
     $/ = $oldSep;       # restore input line separator
     
     # erase .failed file if test was successful
-    $success and unlink $testfile;
+    $success and not $keep and unlink $testfile;
     
     return $success
 }
@@ -151,8 +152,8 @@ sub closeEnough($$)
         my $tok2 = $toks2[$i];
         last unless defined $tok1 and defined $tok2;
         next if $tok1 eq $tok2;
-        # can't compare any more if either line was truncated (ie. ends with '[...]')
-        return $lenChanged if $tok1 =~ /\Q[...]\E$/ or $tok2 =~ /\Q[...]\E$/;
+        # can't compare any more if either line was truncated (ie. ends with '[...]' or '[snip]')
+        return $lenChanged if $tok1 =~ /\[(\.{3}|snip)\]$/ or $tok2 =~ /\[(\.{3}|snip)\]$/;
         # account for different timezones
         if ($tok1 =~ /^(\d{2}:\d{2}:\d{2})(Z|[-+]\d{2}:\d{2})$/i) {
             my $time = $1;  # remove timezone
@@ -160,12 +161,22 @@ sub closeEnough($$)
             next if $tok2 =~ /^(\d{2}:\d{2}:\d{2})(Z|[-+]\d{2}:\d{2})$/i and $time eq $1;
             # date/time may be wrong to if converting GMT value to local time
             last unless $i and $toks1[$i-1] =~ /^\d{4}:\d{2}:\d{2}$/ and
-                               $toks1[$i-1] =~ /^\d{4}:\d{2}:\d{2}$/;
+                               $toks2[$i-1] =~ /^\d{4}:\d{2}:\d{2}$/;
             $tok1 = $toks1[$i-1] . ' ' . $tok1; # add date to give date/time value
             $tok2 = $toks2[$i-1] . ' ' . $tok2;
+CheckTimeDifference:
             my $t1 = Image::ExifTool::GetUnixTime($tok1, 'local') or last;
             my $t2 = Image::ExifTool::GetUnixTime($tok2, 'local') or last;
-            last unless $t1 == $t2;
+            my $td = $t2 - $t1;
+            if ($td) {
+                # patch for the MirBSD leap-second unconformity
+                # (120 leap seconds should cover us until _well_ into the future)
+                last unless $^O eq 'mirbsd' and $td < 0 and $td > -120;
+                warn "\n  Ignoring $td second error due to MirBSD leap-second \"feature\":\n";
+                chomp($line1,$line2);
+                warn qq{    Test gave: "$line2"\n};
+                warn qq{    Should be: "$line1"\n};
+            }
         # date may be different if timezone shifted into next day
         } elsif ($tok1 =~ /^\d{4}:\d{2}:\d{2}$/ and $tok2 =~ /^\d{4}:\d{2}:\d{2}$/ and
                  defined $toks1[$i+1] and defined $toks2[$i+1] and
@@ -173,11 +184,9 @@ sub closeEnough($$)
                  $toks2[$i+1] =~ /^(\d{2}:\d{2}:\d{2})(Z|[-+]\d{2}:\d{2})$/i)
         {
             ++$i;
-            $tok1 .= ' ' . $toks1[$i];          # add time to give date/time value
+            $tok1 .= ' ' . $toks1[$i];      # add time to give date/time value
             $tok2 .= ' ' . $toks2[$i];
-            my $t1 = Image::ExifTool::GetUnixTime($tok1, 'local') or last;
-            my $t2 = Image::ExifTool::GetUnixTime($tok2, 'local') or last;
-            last unless $t1 == $t2;
+            goto CheckTimeDifference;   # GOTO!
         } else {
             # check to see if both tokens are floating point numbers (with decimal points!)
             if ($tok1 =~ s/([^\d.]+)$//) {  # remove trailing units
@@ -235,14 +244,14 @@ sub formatValue($)
         foreach (@$val) {
             push @a, formatValue($_);
         }
-        $str = join(', ', @a);
+        $str = '[' . join(',', @a) . ']';
     } elsif (ref $val eq 'HASH') {
         my $key;
         foreach $key (sort keys %$val) {
             push @a, $key . '=' . formatValue($$val{$key});
         }
-        $str = '{ ' . join('; ',@a) . ' }';
-    } else {
+        $str = '{' . join(',', @a) . '}';
+    } elsif (defined $val) {
         # make sure there are no linefeeds in output
         ($str = $val) =~ tr/\x0a\x0d/;/;
         # translate unknown characters
@@ -250,6 +259,8 @@ sub formatValue($)
         $str =~ tr/\x01-\x1f\x7f/./;
         # remove NULL chars
         $str =~ s/\x00//g;
+    } else {
+        $str = '';
     }
     return $str;
 }
@@ -309,14 +320,15 @@ sub check($$$;$$$)
 }
 
 #------------------------------------------------------------------------------
-# test writing feature by writing specified information to JPEG file
+# Test writing feature by writing specified information to JPEG file
 # Inputs: 0) list reference to lists of SetNewValue arguments
 #         1) test name, 2) test number, 3) optional source file name,
 #         4) true to only check tags which were written (or list ref for tags to check)
+#         5) flag set if nothing is expected to change in the output file
 # Returns: 1 if check passed
-sub writeCheck($$$;$$)
+sub writeCheck($$$;$$$)
 {
-    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten) = @_;
+    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten, $same) = @_;
     $srcfile or $srcfile = "t/images/$testname.jpg";
     my ($ext) = ($srcfile =~ /\.(.+?)$/);
     my $testfile = "t/${testname}_${testnum}_failed.$ext";
@@ -331,17 +343,39 @@ sub writeCheck($$$;$$)
         push @tags, $$_[0] if $onlyWritten;
     }
     unlink $testfile;
-    $exifTool->WriteInfo($srcfile, $testfile);
-    my $info = $exifTool->GetInfo('Error');
-    foreach (keys %$info) { warn "$$info{$_}\n"; }
-    $info = $exifTool->ImageInfo($testfile,{Duplicates=>1,Unknown=>1},@tags);
+    my $ok = writeInfo($exifTool, $srcfile, $testfile, $same);
+    my $info = $exifTool->ImageInfo($testfile,{Duplicates=>1,Unknown=>1},@tags);
     my $rtnVal = check($exifTool, $info, $testname, $testnum);
-    $rtnVal and unlink $testfile;
-    return $rtnVal;
+    return 0 unless $ok and $rtnVal;
+    unlink $testfile;
+    return 1;
 }
 
 #------------------------------------------------------------------------------
-# test verbose output
+# Call Image::ExifTool::WriteInfo with error checking
+# Inputs: 0) ExifTool ref, 1) src file, 2) dst file, 3) true if nothing should change
+# Return: true on success
+sub writeInfo($$;$$)
+{
+    my ($exifTool, $src, $dst, $same) = @_;
+    # erase temporary file created by WriteInfo() if no destination file is given
+    # (may be left over from previous crashed tests)
+    unlink "${src}_exiftool_tmp" if not defined $dst and not ref $src;
+    my $result = $exifTool->WriteInfo($src, $dst);
+    my $err = '';
+    $err .= "  Error: WriteInfo() returned $result\n" if $result != ($same ? 2 : 1);
+    my $info = $exifTool->GetInfo('Warning', 'Error');
+    foreach (sort keys %$info) {
+        my $tag = Image::ExifTool::GetTagName($_);
+        $err .= "  $tag: $$info{$_}\n";
+    }
+    return 1 unless $err;
+    warn "\n$err";
+    return 0;
+}
+
+#------------------------------------------------------------------------------
+# Test verbose output
 # Inputs: 0) test name, 1) test number, 2) Input file, 3) verbose level
 # Returns: true if test passed
 sub testVerbose($$$$)
